@@ -1,3 +1,4 @@
+//go:build !windows
 // +build !windows
 
 /*
@@ -20,6 +21,7 @@ package shim
 
 import (
 	"context"
+	"fmt"
 	"io"
 	"net"
 	"os"
@@ -28,7 +30,6 @@ import (
 
 	"github.com/containerd/containerd/sys/reaper"
 	"github.com/containerd/fifo"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 )
@@ -59,7 +60,7 @@ func serveListener(path string) (net.Listener, error) {
 		path = "[inherited from parent]"
 	} else {
 		if len(path) > socketPathLimit {
-			return nil, errors.Errorf("%q: unix socket path too long (> %d)", path, socketPathLimit)
+			return nil, fmt.Errorf("%q: unix socket path too long (> %d)", path, socketPathLimit)
 		}
 		l, err = net.Listen("unix", path)
 	}
@@ -70,7 +71,7 @@ func serveListener(path string) (net.Listener, error) {
 	return l, nil
 }
 
-func handleSignals(ctx context.Context, logger *logrus.Entry, signals chan os.Signal) error {
+func reap(ctx context.Context, logger *logrus.Entry, signals chan os.Signal) error {
 	logger.Info("starting signal loop")
 
 	for {
@@ -78,6 +79,8 @@ func handleSignals(ctx context.Context, logger *logrus.Entry, signals chan os.Si
 		case <-ctx.Done():
 			return ctx.Err()
 		case s := <-signals:
+			// Exit signals are handled separately from this loop
+			// They get registered with this channel so that we can ignore such signals for short-running actions (e.g. `delete`)
 			switch s {
 			case unix.SIGCHLD:
 				if err := reaper.Reap(); err != nil {
@@ -89,6 +92,22 @@ func handleSignals(ctx context.Context, logger *logrus.Entry, signals chan os.Si
 	}
 }
 
+func handleExitSignals(ctx context.Context, logger *logrus.Entry, cancel context.CancelFunc) {
+	ch := make(chan os.Signal, 32)
+	signal.Notify(ch, syscall.SIGINT, syscall.SIGTERM)
+
+	for {
+		select {
+		case s := <-ch:
+			logger.WithField("signal", s).Debugf("Caught exit signal")
+			cancel()
+			return
+		case <-ctx.Done():
+			return
+		}
+	}
+}
+
 func openLog(ctx context.Context, _ string) (io.Writer, error) {
 	return fifo.OpenFifoDup2(ctx, "log", unix.O_WRONLY, 0700, int(os.Stderr.Fd()))
 }
