@@ -1,3 +1,4 @@
+//go:build !windows
 // +build !windows
 
 /*
@@ -20,8 +21,10 @@ package v2
 
 import (
 	"context"
+	"errors"
 	"io"
 	"net"
+	"os"
 	"path/filepath"
 	"time"
 
@@ -34,12 +37,9 @@ func openShimLog(ctx context.Context, bundle *Bundle, _ func(string, time.Durati
 }
 
 func checkCopyShimLogError(ctx context.Context, err error) error {
-	// When using a multi-container shim, the fifo of the 2nd to Nth
-	// container will not be opened when the ctx is done. This will
-	// cause an ErrReadClosed that can be ignored.
 	select {
 	case <-ctx.Done():
-		if err == fifo.ErrReadClosed {
+		if err == fifo.ErrReadClosed || errors.Is(err, os.ErrClosed) {
 			return nil
 		}
 	default:
