@@ -18,7 +18,7 @@ package native
 
 import (
 	"context"
-	"io/ioutil"
+	"fmt"
 	"os"
 	"path/filepath"
 
@@ -28,7 +28,6 @@ import (
 	"github.com/containerd/containerd/snapshots/storage"
 
 	"github.com/containerd/continuity/fs"
-	"github.com/pkg/errors"
 )
 
 type snapshotter struct {
@@ -138,7 +137,7 @@ func (o *snapshotter) Mounts(ctx context.Context, key string) ([]mount.Mount, er
 	s, err := storage.GetSnapshot(ctx, key)
 	t.Rollback()
 	if err != nil {
-		return nil, errors.Wrap(err, "failed to get snapshot mount")
+		return nil, fmt.Errorf("failed to get snapshot mount: %w", err)
 	}
 	return o.mounts(s), nil
 }
@@ -151,11 +150,17 @@ func (o *snapshotter) Commit(ctx context.Context, name, key string, opts ...snap
 
 	id, _, _, err := storage.GetInfo(ctx, key)
 	if err != nil {
+		if rerr := t.Rollback(); rerr != nil {
+			log.G(ctx).WithError(rerr).Warn("failed to rollback transaction")
+		}
 		return err
 	}
 
 	usage, err := fs.DiskUsage(ctx, o.getSnapshotDir(id))
 	if err != nil {
+		if rerr := t.Rollback(); rerr != nil {
+			log.G(ctx).WithError(rerr).Warn("failed to rollback transaction")
+		}
 		return err
 	}
 
@@ -163,7 +168,7 @@ func (o *snapshotter) Commit(ctx context.Context, name, key string, opts ...snap
 		if rerr := t.Rollback(); rerr != nil {
 			log.G(ctx).WithError(rerr).Warn("failed to rollback transaction")
 		}
-		return errors.Wrap(err, "failed to commit snapshot")
+		return fmt.Errorf("failed to commit snapshot: %w", err)
 	}
 	return t.Commit()
 }
@@ -185,14 +190,14 @@ func (o *snapshotter) Remove(ctx context.Context, key string) (err error) {
 
 	id, _, err := storage.Remove(ctx, key)
 	if err != nil {
-		return errors.Wrap(err, "failed to remove")
+		return fmt.Errorf("failed to remove: %w", err)
 	}
 
 	path := o.getSnapshotDir(id)
 	renamed := filepath.Join(o.root, "snapshots", "rm-"+id)
 	if err := os.Rename(path, renamed); err != nil {
 		if !os.IsNotExist(err) {
-			return errors.Wrap(err, "failed to rename")
+			return fmt.Errorf("failed to rename: %w", err)
 		}
 		renamed = ""
 	}
@@ -203,10 +208,10 @@ func (o *snapshotter) Remove(ctx context.Context, key string) (err error) {
 		if renamed != "" {
 			if err1 := os.Rename(renamed, path); err1 != nil {
 				// May cause inconsistent data on disk
-				log.G(ctx).WithError(err1).WithField("path", renamed).Errorf("failed to rename after failed commit")
+				log.G(ctx).WithError(err1).WithField("path", renamed).Error("failed to rename after failed commit")
 			}
 		}
-		return errors.Wrap(err, "failed to commit")
+		return fmt.Errorf("failed to commit: %w", err)
 	}
 	if renamed != "" {
 		if err := os.RemoveAll(renamed); err != nil {
@@ -234,23 +239,23 @@ func (o *snapshotter) createSnapshot(ctx context.Context, kind snapshots.Kind, k
 	)
 
 	if kind == snapshots.KindActive || parent == "" {
-		td, err = ioutil.TempDir(filepath.Join(o.root, "snapshots"), "new-")
+		td, err = os.MkdirTemp(filepath.Join(o.root, "snapshots"), "new-")
 		if err != nil {
-			return nil, errors.Wrap(err, "failed to create temp dir")
+			return nil, fmt.Errorf("failed to create temp dir: %w", err)
 		}
 		if err := os.Chmod(td, 0755); err != nil {
-			return nil, errors.Wrapf(err, "failed to chmod %s to 0755", td)
+			return nil, fmt.Errorf("failed to chmod %s to 0755: %w", td, err)
 		}
 		defer func() {
 			if err != nil {
 				if td != "" {
 					if err1 := os.RemoveAll(td); err1 != nil {
-						err = errors.Wrapf(err, "remove failed: %v", err1)
+						err = fmt.Errorf("remove failed: %v: %w", err1, err)
 					}
 				}
 				if path != "" {
 					if err1 := os.RemoveAll(path); err1 != nil {
-						err = errors.Wrapf(err, "failed to remove path: %v", err1)
+						err = fmt.Errorf("failed to remove path: %v: %w", err1, err)
 					}
 				}
 			}
@@ -267,7 +272,7 @@ func (o *snapshotter) createSnapshot(ctx context.Context, kind snapshots.Kind, k
 		if rerr := t.Rollback(); rerr != nil {
 			log.G(ctx).WithError(rerr).Warn("failed to rollback transaction")
 		}
-		return nil, errors.Wrap(err, "failed to create snapshot")
+		return nil, fmt.Errorf("failed to create snapshot: %w", err)
 	}
 
 	if td != "" {
@@ -282,7 +287,10 @@ func (o *snapshotter) createSnapshot(ctx context.Context, kind snapshots.Kind, k
 				fs.WithXAttrErrorHandler(xattrErrorHandler),
 			}
 			if err := fs.CopyDir(td, parent, copyDirOpts...); err != nil {
-				return nil, errors.Wrap(err, "copying of parent failed")
+				if rerr := t.Rollback(); rerr != nil {
+					log.G(ctx).WithError(rerr).Warn("failed to rollback transaction")
+				}
+				return nil, fmt.Errorf("copying of parent failed: %w", err)
 			}
 		}
 
@@ -291,13 +299,13 @@ func (o *snapshotter) createSnapshot(ctx context.Context, kind snapshots.Kind, k
 			if rerr := t.Rollback(); rerr != nil {
 				log.G(ctx).WithError(rerr).Warn("failed to rollback transaction")
 			}
-			return nil, errors.Wrap(err, "failed to rename")
+			return nil, fmt.Errorf("failed to rename: %w", err)
 		}
 		td = ""
 	}
 
 	if err := t.Commit(); err != nil {
-		return nil, errors.Wrap(err, "commit failed")
+		return nil, fmt.Errorf("commit failed: %w", err)
 	}
 
 	return o.mounts(s), nil
