@@ -18,16 +18,14 @@ package storage
 
 import (
 	"context"
+	"errors"
 	"fmt"
-	"io/ioutil"
-	"os"
 	"testing"
 	"time"
 
 	"github.com/containerd/containerd/errdefs"
 	"github.com/containerd/containerd/snapshots"
 	"github.com/google/go-cmp/cmp"
-	"github.com/pkg/errors"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 )
@@ -67,17 +65,16 @@ func MetaStoreSuite(t *testing.T, name string, meta func(root string) (*MetaStor
 func makeTest(t *testing.T, name string, metaFn metaFactory, fn testFunc) func(t *testing.T) {
 	return func(t *testing.T) {
 		ctx := context.Background()
-		tmpDir, err := ioutil.TempDir("", "metastore-test-"+name+"-")
-		if err != nil {
-			t.Fatal(err)
-		}
-		defer os.RemoveAll(tmpDir)
 
-		ms, err := metaFn(tmpDir)
+		ms, err := metaFn(t.TempDir())
 		if err != nil {
 			t.Fatal(err)
 		}
 
+		t.Cleanup(func() {
+			ms.Close()
+		})
+
 		fn(ctx, t, ms)
 	}
 }
@@ -148,31 +145,31 @@ func inWriteTransaction(fn testFunc) testFunc {
 // - "active-5": readonly active with parent "committed-2"
 func basePopulate(ctx context.Context, ms *MetaStore) error {
 	if _, err := CreateSnapshot(ctx, snapshots.KindActive, "committed-tmp-1", ""); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CommitActive(ctx, "committed-tmp-1", "committed-1", snapshots.Usage{Size: 1}); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CreateSnapshot(ctx, snapshots.KindActive, "committed-tmp-2", "committed-1"); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CommitActive(ctx, "committed-tmp-2", "committed-2", snapshots.Usage{Size: 2}); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CreateSnapshot(ctx, snapshots.KindActive, "active-1", ""); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CreateSnapshot(ctx, snapshots.KindActive, "active-2", "committed-1"); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CreateSnapshot(ctx, snapshots.KindActive, "active-3", "committed-2"); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CreateSnapshot(ctx, snapshots.KindView, "view-1", ""); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	if _, err := CreateSnapshot(ctx, snapshots.KindView, "view-2", "committed-2"); err != nil {
-		return errors.Wrap(err, "failed to create active")
+		return fmt.Errorf("failed to create active: %w", err)
 	}
 	return nil
 }
@@ -274,7 +271,7 @@ func testWalk(ctx context.Context, t *testing.T, _ *MetaStore) {
 	found := map[string]snapshots.Info{}
 	err := WalkInfo(ctx, func(ctx context.Context, info snapshots.Info) error {
 		if _, ok := found[info.Name]; ok {
-			return errors.Errorf("entry already encountered")
+			return errors.New("entry already encountered")
 		}
 		found[info.Name] = info
 		return nil
@@ -287,10 +284,10 @@ func testGetSnapshot(ctx context.Context, t *testing.T, ms *MetaStore) {
 	snapshotMap := map[string]Snapshot{}
 	populate := func(ctx context.Context, ms *MetaStore) error {
 		if _, err := CreateSnapshot(ctx, snapshots.KindActive, "committed-tmp-1", ""); err != nil {
-			return errors.Wrap(err, "failed to create active")
+			return fmt.Errorf("failed to create active: %w", err)
 		}
 		if _, err := CommitActive(ctx, "committed-tmp-1", "committed-1", snapshots.Usage{}); err != nil {
-			return errors.Wrap(err, "failed to create active")
+			return fmt.Errorf("failed to create active: %w", err)
 		}
 
 		for _, opts := range []struct {
@@ -319,7 +316,7 @@ func testGetSnapshot(ctx context.Context, t *testing.T, ms *MetaStore) {
 		} {
 			active, err := CreateSnapshot(ctx, opts.Kind, opts.Name, opts.Parent)
 			if err != nil {
-				return errors.Wrap(err, "failed to create active")
+				return fmt.Errorf("failed to create active: %w", err)
 			}
 			snapshotMap[opts.Name] = active
 		}
